package info.bukova.isspst.services.approved;

import info.bukova.isspst.dao.BaseDao;
import info.bukova.isspst.data.JoinedItem;
import info.bukova.isspst.data.RequirementItem;
import info.bukova.isspst.data.RequirementState;
import info.bukova.isspst.data.Workgroup;
import info.bukova.isspst.services.AbstractService;
import info.bukova.isspst.services.users.UserService;
import info.bukova.isspst.services.workgroups.WorkgroupService;

import java.util.ArrayList;
import java.util.List;

import org.hibernate.Query;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PostFilter;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;

public class ApprovedServiceImpl extends AbstractService<JoinedItem> implements ApprovedService
{
	@Autowired
	WorkgroupService workgroupService;

	@Autowired
	UserService userService;

	// V bázi je BaseDao<T>, což nelze použít, protože třída JoinedItem nemá
	// ekvivalent v databázi
	// Bylo by dobré tuto funčnost vložit do báze...
	BaseDao<?> dao4Query;

	public BaseDao<?> getDao4Query()
	{
		return dao4Query;
	}

	public void setDao4Query(BaseDao<?> dao4Query)
	{
		this.dao4Query = dao4Query;
	}
	
	@SuppressWarnings("unchecked")
	@Override
	@Transactional
	@PreAuthorize("hasPermission(this, 'PERM_SHOW_CENTRE_REQ')")
	@PostFilter("hasPermission(filterObject, 'PERM_SHOW_CENTRE_REQ')")
	public List<JoinedItem> getAll()
	{
		List<Workgroup> wgList = workgroupService.getUserCentres(userService.getCurrent());
		Query q = dao4Query.getQuery("select item from RequirementItem item left join item.requirement rq join rq.centre c where rq.kind is not null and rq.state = :state and c in (:wgList)");
		q.setParameterList("wgList", wgList);		
		q.setParameter("state", RequirementState.APPROVED);
		List<JoinedItem> items = new ArrayList<JoinedItem>();

		for (RequirementItem it : (List<RequirementItem>)q.list()) {
			items.add(new JoinedItem(it, it.getRequirement().getWorkgroup(), it.getRequirement().getCentre(), it.getRequirement().getOwnedBy()));
		}
		
		return items;
	}
}
